//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Descriptor/DoubleProperty.cpp
//! @brief     Implements class DoubleProperty
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Descriptor/DoubleProperty.h"
#include "GUI/Support/XML/Serialize.h"
#include "GUI/Support/XML/UtilXML.h"
#include <QUuid>

using std::variant;

void DoubleProperty::init(const QString& label, const QString& tooltip, double value,
                          const variant<QString, Unit>& unit, const QString& uidPrefix)
{
    init(label, tooltip, value, unit, 3, RealLimits::nonnegative(), uidPrefix);
}

void DoubleProperty::init(const QString& label, const QString& tooltip, double value,
                          const variant<QString, Unit>& unit, uint decimals,
                          const RealLimits& limits, const QString& uidPrefix)
{
    init(label, tooltip, value, unit, decimals, 0.01, limits, uidPrefix);
}

void DoubleProperty::init(const QString& label, const QString& tooltip, double value,
                          const std::variant<QString, Unit>& unit, uint decimals, double step,
                          const RealLimits& limits, const QString& uidPrefix)
{
    m_label = label;
    m_tooltip = tooltip;
    m_value = value;
    m_decimals = decimals;
    m_step = step;
    m_limits = limits;

    if(uidPrefix.size()>0)
        m_uid = uidPrefix + "/" + QUuid::createUuid().toString();
    else
        m_uid = QUuid::createUuid().toString();

    // temporary
    m_descriptor.label = label;
    m_descriptor.tooltip = tooltip;
    m_descriptor.decimals = decimals;
    m_descriptor.limits = limits;
    m_descriptor.unit = unit;
    m_descriptor.set = [=](double v) { m_value = v; };
    m_descriptor.get = [=] { return m_value; };
    m_descriptor.path = [=] { return m_uid; };
}

void DoubleProperty::setTooltip(const QString& tooltip)
{
    m_descriptor.tooltip = tooltip;
}

void DoubleProperty::setUnit(const variant<QString, Unit>& unit)
{
    m_descriptor.unit = unit;
}

void DoubleProperty::setLimits(const RealLimits& limits)
{
    m_descriptor.limits = limits;
}

bool DoubleProperty::isInitialized() const
{
    return !m_uid.isEmpty();
}

void Serialize::rwProperty(Streamer& s, const QString& tag, DoubleProperty& d)
{
    if (QXmlStreamWriter* w = s.xmlWriter()) {
        w->writeStartElement(tag);
        GUI::Session::XML::writeAttribute(w, XML::Tags::Value, d.get());
        w->writeAttribute(XML::Tags::Id, d.uid());
        GUI::Session::XML::writeAttribute(w, XML::Tags::Decimals, d.decimals());
        w->writeEndElement();
    } else if (QXmlStreamReader* r = s.xmlReader()) {
        r->readNextStartElement();
        s.assertCurrentTag(tag);
        d.set(r->attributes().value(XML::Tags::Value).toDouble());
        d.setUid(r->attributes().value(XML::Tags::Id).toString());
        d.setDecimals(r->attributes().value(XML::Tags::Decimals).toUInt());
        s.gotoEndElementOfTag(tag);
    } else
        ASSERT(0);
}
